;;; editor/meow/config.el -*- lexical-binding: t; -*-

;; Setup Functions

;; Leader Key
(defun meow/setup-leader ()
  (map! :leader
        :desc "Meow Cheatsheet"
        "?" #'meow-cheatsheet))

(defun meow/setup-qwerty ()
  (setq meow-cheatsheet-layout meow-cheatsheet-layout-qwerty)
  (meow-motion-overwrite-define-key
   '("j" . meow-next)
   '("k" . meow-prev))
  (meow-leader-define-key
   ;; SPC j/k will run the original command in MOTION state.
   '("j" . "H-j")
   '("k" . "H-k"))

  ;; (map! :map meow-paren-state-keymap
  ;;       "h" #'sp-backward-sexp
  ;;       "j" #'sp-forward-sexp)

  ;; (meow-define-state paren
  ;;   "meow state for lisps"
  ;;   :lighter " [P]"
  ;;   :keymap meow-paren-state-keymap)

  (map! :map meow-normal-state-keymap
        "<" #'meow-beginning-of-thing
        ">" #'meow-end-of-thing
        "a" #'meow-append
        "A" #'meow-open-below
        "b" #'meow-back-word
        "B" #'meow-back-symbol
        "c" #'meow-change
        "d" #'meow-delete
        "D" #'meow-backward-delete
        "e" #'meow-next-word
        "E" #'meow-next-symbol
        "f" #'meow-find
        "g" #'meow-cancel-selection
        "G" #'meow-grab
        "h" #'meow-left
        "H" #'meow-left-expand
        "i" #'meow-insert
        "I" #'meow-open-above
        "j" #'meow-next
        "J" #'meow-next-expand
        "k" #'meow-prev
        "K" #'meow-prev-expand
        "l" #'meow-right
        "L" #'meow-right-expand
        "m" #'meow-join
        "n" #'meow-search
        "o" #'meow-block
        "O" #'meow-to-block
        "p" #'meow-yank
        "q" #'meow-quit
        "Q" #'meow-goto-line
        "r" #'meow-replace
        "R" #'meow-swap-grab
        "s" #'meow-kill
        "t" #'meow-till
        "u" #'meow-undo
        "U" #'meow-undo-in-selection
        "v" #'meow-visit
        "w" #'meow-mark-word
        "W" #'meow-mark-symbol
        "x" #'meow-line
        "X" #'meow-goto-line
        "y" #'meow-save
        "Y" #'meow-sync-grab
        "z" #'meow-pop-selection
        "0" #'meow-expand-0
        "1" #'meow-expand-1
        "2" #'meow-expand-2
        "3" #'meow-expand-3
        "4" #'meow-expand-4
        "5" #'meow-expand-5
        "6" #'meow-expand-6
        "7" #'meow-expand-7
        "8" #'meow-expand-8
        "9" #'meow-expand-9
        "-" #'negative-argument
        ";" #'meow-reverse
        "," #'meow-inner-of-thing
        "." #'meow-bounds-of-thing
        "\\" #'bookmark-jump
        "`" #'bookmark-set
        "~" #'bookmark-delete
        "'" #'repeat
        "=" #'meow--eval-sexp
        ":" #'meow--eval-sexp
        "[" #'sp-backward-sexp
        "]" #'sp-forward-sexp
        "{" #'sp-backward-up-sexp
        "}" #'sp-down-sexp
        "<escape>" #'corfu-quit))

(defun meow--eval-sexp ()
  (interactive)
  (cond ((derived-mode-p 'lisp-mode) (sly-eval-defun))
    	((derived-mode-p 'sclang-mode) (sclang-eval-region-or-line))
	(t (eros-eval-defun nil))))

(defun meow--post-isearch-function ()
  (unless isearch-mode-end-hook-quit
    (when isearch-success
      (let ((beg (car isearch-match-data))
            (end (cadr isearch-match-data)))
        (thread-first
          (meow--make-selection '(select . visit) beg (if isearch-forward end isearch-other-end))
          (meow--select (not isearch-forward)))))))

(add-hook 'isearch-mode-end-hook 'meow--post-isearch-function)

(use-package! meow
  :hook (doom-after-modules-config . meow-global-mode)
  :demand t
  :config
  (meow/setup-qwerty)
  (map! :map meow-normal-state-keymap
        doom-leader-key doom-leader-map)
  (map! :map meow-motion-state-keymap
        doom-leader-key doom-leader-map)
  (map! :map meow-beacon-state-keymap
        doom-leader-key nil)
  (meow/setup-leader)
  (pushnew! meow-mode-state-list '(sly-db-mode . motion))
  (pushnew! meow-mode-state-list '(sly-inspector-mode . motion)))
;; (pushnew! meow-mode-state-list '(custom-mode . motion)))
